import { __decorate } from "tslib";
import { component, css, extractEventHandler, html, nothing, property, style } from '@a11d/lit';
import { Button } from '@3mo/button';
import '@3mo/circular-progress';
/**
 * @attr loading
 * @attr preventClickEventInference
 */
let LoadingButton = class LoadingButton extends Button {
    constructor() {
        super(...arguments);
        this.loading = false;
        this.preventClickEventInference = false;
        this.eventListeners = new Array();
        this.clickHandler = async (e) => {
            if (this.preventClickEventInference === false) {
                const clickEventHandlers = this.eventListeners
                    .filter(({ type }) => type === 'click')
                    .map(({ handler }) => extractEventHandler(handler))
                    .map(handler => handler(e))
                    .filter(Boolean);
                if (clickEventHandlers.length > 0 && this.loading === false) {
                    e.stopImmediatePropagation();
                    this.loading = true;
                    await Promise.allSettled(clickEventHandlers);
                    this.loading = false;
                }
            }
        };
    }
    addEventListener(type, handler, options) {
        this.eventListeners.push({ type, handler });
        super.addEventListener(type, handler, options);
    }
    removeEventListener(type, listener, options) {
        this.eventListeners.forEach(({ handler: eventListener, type }, i) => {
            if (type === type && eventListener === listener) {
                delete this.eventListeners[i];
            }
        });
        super.removeEventListener(type, listener, options);
    }
    initialized() {
        var _a;
        (_a = this.renderRoot.querySelector('mwc-button')) === null || _a === void 0 ? void 0 : _a.addEventListener('click', this.clickHandler);
    }
    disconnected() {
        var _a;
        (_a = this.renderRoot.querySelector('mwc-button')) === null || _a === void 0 ? void 0 : _a.removeEventListener('click', this.clickHandler);
    }
    static get styles() {
        return css `
			${super.styles}
			mwc-button { position: relative; }
		`;
    }
    get contentTemplate() {
        return html `
			${super.contentTemplate}
			${!this.circularProgressReplacesLeadingIcon && this.loading ? this.circularProgressTemplate : nothing}
		`;
    }
    get isDisabled() {
        return super.isDisabled || this.loading;
    }
    get leadingIconTemplate() {
        return this.circularProgressReplacesLeadingIcon && this.loading ? this.circularProgressTemplate : super.leadingIconTemplate;
    }
    get circularProgressReplacesLeadingIcon() {
        return !!this.leadingIcon;
    }
    get circularProgressTemplate() {
        return html `
			<mo-circular-progress ${style({
            position: this.circularProgressReplacesLeadingIcon ? undefined : 'absolute',
            top: this.circularProgressReplacesLeadingIcon ? undefined : '50%',
            insetInlineStart: this.circularProgressReplacesLeadingIcon ? undefined : '50%',
            transform: this.circularProgressReplacesLeadingIcon ? undefined : getComputedStyle(this).direction === 'rtl' ? 'translate(+50%, -50%)' : 'translate(-50%, -50%)',
            width: this.circularProgressReplacesLeadingIcon ? '24px' : 'auto',
            height: this.circularProgressReplacesLeadingIcon ? '24px' : '75%',
        })}></mo-circular-progress>
		`;
    }
};
__decorate([
    property({ type: Boolean, reflect: true })
], LoadingButton.prototype, "loading", void 0);
__decorate([
    property({ type: Boolean })
], LoadingButton.prototype, "preventClickEventInference", void 0);
LoadingButton = __decorate([
    component('mo-loading-button')
], LoadingButton);
export { LoadingButton };
